import argparse
import sys
import codecs
import os
import MySQLdb

if __name__ == '__main__' and __package__ is None:
    from os import sys, path
    sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))

import kian.fitness
from kian import TrainedModel
from kian import Kian


def main():
    args = sys.argv[1:]
    parser = argparse.ArgumentParser(description='Parse and add statements '
                                     'based on trained model')
    parser.add_argument('--name', '-n', nargs='?', required=True,
                        help='name of the model to train')
    parsed_args = parser.parse_known_args(args)[0]

    model = TrainedModel.from_file(parsed_args.name)
    print('Loading the model')
    model.load_data()
    model.load()
    file_path = os.path.join(model.data_directory, 'res2.dat')

    if not os.path.isfile(file_path):
        raise ValueError('You should train the model first')
    with codecs.open(file_path, 'r', 'utf-8') as f:
        cv_set = eval(f.read())
    first_thrashhold = kian.fitness.optimum_thrashhold(cv_set, 1)[0]
    second_thrashhold = kian.fitness.optimum_thrashhold(cv_set, 0.125)[0]
    print('Finding possible adds to human review')
    print(first_thrashhold, second_thrashhold)
    pros = model.wikidata_data_w | model.wikidata_data_wo
    print(len(pros))
    name = None
    a = []
    db = MySQLdb.connect(host="tools-db", db="s52709__kian_p",
                         read_default_file="~/replica.my.cnf")
    cursor = db.cursor()
    insert_statement = (
        "INSERT INTO kian "
        "(qid, model_name, wiki_name, property, value, status, prob) "
        "VALUES (%s, %s, %s, %s, %s, %s, %s)")
    with model.wiki_data_file as f:
        for line in f:
            line = line.replace('\n', '')
            if u'\t' not in line:
                continue
            if name and name != line.split('\t')[0]:
                if name in pros:
                    a = []
                    name = line.split('\t')[0]
                    continue
                features = model.label_case(a)
                res = Kian.kian(model.theta, features)[0]
                if res > first_thrashhold and res < second_thrashhold:
                    cursor.execute(insert_statement,
                                   (name, model.name, model.wiki,
                                    model.property_name, model.value, 0,
                                    str(res)[:7]))
                a = []
            name = line.split('\t')[0]
            a.append(line.split('\t')[1])
    db.commit()
    cursor.close()
    db.close()


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        sys.exit()



import sys
import MySQLdb
import urllib2


url = "http://tools.wmflabs.org/autolist/index.php?wdq=claim[" \
    "{statemnt}:{value}]&run=Run&download=1"
res_human = urllib2.urlopen(
    url.format(statemnt='31',
               value='5')).read().decode('utf-8')


def main():
    db = MySQLdb.connect(host="tools-db", db="s52709__kian_p",
                         read_default_file="~/replica.my.cnf")
    cursor = db.cursor()
    select_statement = (
        "SELECT property, value from kian "
        "where status = 0 "
        "group by property, value;")
    cursor.execute(select_statement)
    cases = list(cursor.fetchall())
    for case in cases:
        print('Working on {case}'.format(case=case))
        url = "http://tools.wmflabs.org/autolist/index.php?wdq=claim[" \
            "{statemnt}:{value}]&run=Run&download=1"
        if case[1] != 'Q5':
            res = urllib2.urlopen(
                url.format(statemnt=case[0][1:],
                           value=case[1][1:])).read().decode('utf-8')
        else:
            res = res_human
        res = set(res.split('\n'))
        select_statement = (
            "SELECT qid from kian "
            "where status = 0 and property = "
            "'{prop}' and value = '{val}';".format(prop=case[0], val=case[1]))
        cursor.execute(select_statement)
        res2 = set([i[0] for i in cursor.fetchall()])
        intersection = res & res2
        print(len(res), len(res2), len(intersection))
        str_intersection = str(tuple(intersection)).replace('u\'Q', '\'Q')
        set_statement = (
            "UPDATE kian SET status = 1 "
            "where qid in {qid} and property = "
            "'{prop}' and value = '{val}';".format(qid=str_intersection,
                                                   prop=case[0], val=case[1]))
        cursor.execute(set_statement)
    db.commit()
    cursor.close()
    db.close()


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        sys.exit()



import argparse
import pywikibot
import sys
import codecs
import os
if __name__ == '__main__' and __package__ is None:
    from os import sys, path
    sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))

import kian.fitness
from kian import TrainedModel
from kian import Kian

site = pywikibot.Site('wikidata', fam='wikidata')
sources = {
    'enwiki': 'Q328',
    'svwiki': 'Q169514',
    'dewiki': 'Q48183',
    'itwiki': 'Q11920',
    'nowiki': 'Q191769',
    'fawiki': 'Q48952',
    'eswiki': 'Q8449',
    'plwiki': 'Q1551807',
    'cawiki': 'Q199693',
    'frwiki': 'Q8447',
    'nlwiki': 'Q10000',
    'ptwiki': 'Q11921',
    'ruwiki': 'Q206855',
    'hewiki': 'Q199913',
    'viwiki': 'Q200180',
    'bewiki': 'Q877583',
    'ukwiki': 'Q199698',
    'trwiki': 'Q58255',
    'cswiki': 'Q191168',
    'shwiki': 'Q58679',
    'jawiki': 'Q177837',
    'kowiki': 'Q17985',
    'fiwiki': 'Q175482',
    'huwiki': 'Q53464',
    'rowiki': 'Q199864',
    'zhwiki': 'Q30239',
    'ltwiki': 'Q202472',
    'srwiki': 'Q200386',
    'arwiki': 'Q199700',
    'skwiki': 'Q192582',
    'elwiki': 'Q11918',
    'hrwiki': 'Q203488',
    'mswiki': 'Q845993',
    'glwiki': 'Q841208',
    'euwiki': 'Q207260'}
repo = site.data_repository()


def main():
    args = sys.argv[1:]
    parser = argparse.ArgumentParser(description='Parse and add statements '
                                     'based on trained model')
    parser.add_argument('--name', '-n', nargs='?', required=True,
                        help='name of the model to train')
    parsed_args = parser.parse_known_args(args)[0]

    model = TrainedModel.from_file(parsed_args.name)
    print('Loading the model')
    model.load_data()
    model.load()
    file_path = os.path.join(model.data_directory, 'res2.dat')

    if not os.path.isfile(file_path):
        raise ValueError('You should train the model first')
    with codecs.open(file_path, 'r', 'utf-8') as f:
        cv_set = eval(f.read())
    second_thrashhold = kian.fitness.optimum_thrashhold(cv_set, 0.125)[0]
    print('Finding possible adds')
    print(second_thrashhold)
    pros = model.wikidata_data_w | model.wikidata_data_wo
    print(len(pros))
    name = None
    a = []
    with model.wiki_data_file as f:
        for line in f:
            line = line.replace('\n', '')
            if u'\t' not in line:
                continue
            if name and name != line.split('\t')[0]:
                if name in pros:
                    a = []
                    name = line.split('\t')[0]
                    continue
                features = model.label_case(a)
                res = Kian.kian(model.theta, features)[0]
                if res < second_thrashhold:
                    a = []
                    continue
                print('Found something on %s, %s' % (name, res))
                item = pywikibot.ItemPage(repo, name)
                try:
                    item.get()
                except:
                    a = []
                    name = line.split('\t')[0]
                    continue
                if model.property_name in item.claims:
                    a = []
                    continue
                print('Adding statements')
                claim = pywikibot.Claim(repo, model.property_name)
                claim.setTarget(pywikibot.ItemPage(repo, model.value))
                summary = ('Bot: Adding %s:%s from %s ([[User:Ladsgroup'
                           '/Kian|Powered by Kian]])' %
                           (model.property_name, model.value, model.wiki))
                try:
                    item.addClaim(claim, summary=summary)
                    source = pywikibot.Claim(repo, 'P143')
                    source.setTarget(
                        pywikibot.ItemPage(repo, sources[model.wiki]))
                    claim.addSource(source)
                except pywikibot.data.api.APIError:
                    continue
                a = []
            name = line.split('\t')[0]
            a.append(line.split('\t')[1])
if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        sys.exit()



import argparse
# TODO: Use time

if __name__ == '__main__' and __package__ is None:
    from os import sys, path
    sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))

from kian import ModelWithData
import kian


parser = argparse.ArgumentParser(description='Train a model.')
parser.add_argument('--name', '-n', nargs='?', required=True,
                    help='name of the model to train')
parser.add_argument('--reload-wikidata', '-rwd', action='store_true',
                    help='Reload data from Wikidata')
parser.add_argument('--reload-wikipedia', '-rwp', action='store_true',
                    help='Reload data from Wikipedia')

args = parser.parse_args()

model = ModelWithData.from_file(args.name)
print('Loading data')
model.load_data(reload_wiki=args.reload_wikipedia,
                reload_wikidata=args.reload_wikidata)
model.retrieve_data(reload_wiki=args.reload_wikipedia,
                    reload_wikidata=args.reload_wikidata)
print('Building the training set')
model.label_categories()
print('Training the model')
bot = kian.Kian(model)
bot.run()



from .model import Model

try:
    basestring
except NameError:
    import urllib.request as urllib2
    import pymysql as MySQLdb
else:
    import urllib2
    import MySQLdb

import codecs
import os
import json


class ModelWithData(Model):
    """docstring for ModelWithData"""
    def __init__(self, *args, **kwargs):
        super(ModelWithData, self).__init__(*args, **kwargs)

    def retrieve_statements_w(self):
        url = "http://tools.wmflabs.org/autolist/index.php?wdq=claim[" \
            "{statemnt}:{value}]_and_link[{wiki}]&run=Run&download=1"
        res = urllib2.urlopen(
            url.format(statemnt=self.property_name[1:],
                       value=self.value[1:],
                       wiki=self.wiki)).read().decode('utf-8')
        return set(res.split('\n'))

    def retrieve_statements_wo(self):
        url = "http://tools.wmflabs.org/autolist/index.php?wdq=claim[" \
            "{statemnt}]_and_noclaim[{statemnt}:{value}]_and_link[{wiki}]" \
            "&run=Run&download=1"
        res = urllib2.urlopen(
            url.format(statemnt=self.property_name[1:],
                       value=self.value[1:],
                       wiki=self.wiki)).read().decode('utf-8')
        res_set = set(res.split('\n'))
        if hasattr(self, 'bias_unit'):
            url = "http://tools.wmflabs.org/autolist/index.php?wdq={bias}" \
                  "_and_noclaim[{statemnt}:{value}]_and_link[{wiki}]" \
                  "&run=Run&download=1"
            res = urllib2.urlopen(
                url.format(statemnt=self.property_name[1:],
                           value=self.value[1:], bias=self.bias_unit,
                           wiki=self.wiki)).read().decode('utf-8')
        return set(res.split('\n')) | res_set

    def sql_query(self, cnf_file='~/replica.my.cnf', host='labsdb'):
        query = "SELECT pp_value, cl_to FROM page_props JOIN categorylinks " \
            "ON pp_page = cl_from JOIN page ON cl_from = page_id WHERE " \
            "pp_propname = 'wikibase_item' AND page_namespace=0 AND " \
            "page_is_redirect=0;"
        conn = MySQLdb.connect('%s.%s' % (self.wiki, host),
                               db='%s_p' % self.wiki,
                               read_default_file=cnf_file,
                               charset='utf8',
                               use_unicode=True)
        cursor = conn.cursor()
        print('Executing the query')
        cursor.execute(query)
        return cursor.fetchall()

    def sql_query2(self, cnf_file='~/replica.my.cnf', host='labsdb'):
        query = "SELECT cl_to, count(*) FROM categorylinks GROUP BY cl_to;"
        conn = MySQLdb.connect('%s.%s' % (self.wiki, host),
                               db='%s_p' % self.wiki,
                               read_default_file=cnf_file,
                               charset='utf8',
                               use_unicode=True)
        cursor = conn.cursor()
        print('Executing the query')
        cursor.execute(query)
        return cursor.fetchall()

    def load_cats(self, wiki_path):
        self.categories = {}
        if os.path.isfile(wiki_path):
            with codecs.open(wiki_path, 'r', 'utf-8') as f:
                for line in f:
                    line = line.replace('\n', '').split('\t')
                    if len(line) != 2:
                        continue
                    self.categories[line[0]] = [int(line[1]), 0, 0]

    def load_data(self, reload_wiki=False, reload_wikidata=False,
                  load_cats=True):
        if not os.path.isdir(self.data_directory):
            os.makedirs(self.data_directory)
        wiki_data_dir = os.path.join(self.data_directory, os.pardir,
                                     '.wiki_data')
        if not os.path.isdir(wiki_data_dir):
            os.makedirs(wiki_data_dir)
        wiki_path = os.path.join(wiki_data_dir, self.wiki + '.dat')
        if os.path.isfile(wiki_path) and not reload_wiki:
            self.wiki_data_file = codecs.open(wiki_path, 'r', 'utf-8')
        wiki_path = os.path.join(wiki_data_dir, self.wiki + '2.dat')
        if load_cats and not reload_wiki:
            self.load_cats(wiki_path)
        file_path = os.path.join(self.data_directory, 'with_statemnts.dat')
        if os.path.isfile(file_path) and not reload_wikidata:
            with codecs.open(file_path, 'r', 'utf-8') as f:
                self.wikidata_data_w = set(f.read().split('\n'))
        file_path = os.path.join(self.data_directory, 'without_statemnts.dat')
        if os.path.isfile(file_path) and not reload_wikidata:
            with codecs.open(file_path, 'r', 'utf-8') as f:
                self.wikidata_data_wo = set(f.read().split('\n'))

    def retrieve_data(self, reload_wiki=False, reload_wikidata=False):
        if not os.path.isdir(self.data_directory):
            os.makedirs(self.data_directory)
        if reload_wiki or not hasattr(self, 'wiki_data'):
            wiki_data_dir = os.path.join(self.data_directory, os.pardir,
                                         '.wiki_data')
            wiki_path = os.path.join(wiki_data_dir, self.wiki + '.dat')
            if not os.path.isfile(wiki_path):
                with codecs.open(wiki_path, 'w', 'utf-8') as f:
                    res = u''
                    for case in self.sql_query():
                        res += u'\t'.join([i.decode('utf-8') for i in case]) \
                            + '\n'
                    f.write(res)
            wiki_path = os.path.join(wiki_data_dir, self.wiki + '2.dat')
            if not os.path.isfile(wiki_path):
                with codecs.open(wiki_path, 'w', 'utf-8') as f:
                    res = u''
                    for case in self.sql_query2():
                        sql_row_res = [str(i).decode('utf-8') for i in case]
                        res += u'\t'.join(sql_row_res) + '\n'
                    f.write(res)
            self.categories = {}
            with codecs.open(wiki_path, 'r', 'utf-8') as f:
                for line in f:
                    line = line.replace('\n', '').split('\t')
                    if len(line) != 2:
                        continue
                    self.categories[line[0]] = [int(line[1]), 0, 0]

        if reload_wikidata or not hasattr(self, 'wikidata_data_w'):
            file_path = os.path.join(
                self.data_directory, 'with_statemnts.dat')
            self.wikidata_data_w = self.retrieve_statements_w()
            with codecs.open(file_path, 'w', 'utf-8') as f:
                f.write(u'\n'.join(self.wikidata_data_w))
            self.wikidata_data_w = set(self.wikidata_data_w)
            file_path = os.path.join(
                self.data_directory, 'without_statemnts.dat')
            self.wikidata_data_wo = self.retrieve_statements_wo()
            with codecs.open(file_path, 'w', 'utf-8') as f:
                f.write(u'\n'.join(self.wikidata_data_wo))
            self.wikidata_data_wo = set(self.wikidata_data_wo)

    def _check(self, name, list_of_cats):
        if name in self.wikidata_data_w:
            for cat in list_of_cats:
                if cat not in self.categories:
                    continue
                self.categories[cat][1] += 1
        elif name in self.wikidata_data_wo:
            for cat in list_of_cats:
                if cat not in self.categories:
                    continue
                self.categories[cat][2] += 1

    def label_case(self, list_of_cats):
        percents = [0, 10, 20, 30, 40, 50, 60, 70, 80, 90]
        x1 = [0] * len(percents)
        x2 = [0] * len(percents)
        for cat in list_of_cats:
            if cat not in self.categories:
                continue
            for i in range(len(percents)):
                percent1 = self.categories[cat][1] * 100.0 \
                    / self.categories[cat][0]
                percent2 = self.categories[cat][2] * 100.0 \
                    / self.categories[cat][0]
                upper = percents[i]
                lower = percents[i - 1]
                if i == 0:
                    if percent1 >= lower:
                        x1[i] += 1
                    if percent2 >= lower:
                        x2[i] += 1
                if percent1 >= lower and percent1 < upper:
                    x1[i] += 1
                if percent2 >= lower and percent2 < upper:
                    x2[i] += 1
        return x1 + x2

    def build_training_set(self, name, list_of_cats):
        y = None
        if len(self.training_set) > 30000:
            return
        if name in self.wikidata_data_w:
            y = [1]
        elif name in self.wikidata_data_wo:
            y = [0]
        else:
            return
        x = self.label_case(list_of_cats)
        self.training_set.add(tuple(x + y))

    def label_categories(self):
        if not (hasattr(self, 'wiki_data_file') and
                hasattr(self, 'wikidata_data_w') and
                hasattr(self, 'wikidata_data_wo') and
                hasattr(self, 'categories')):
            raise ValueError('You need to define enough data to label')

        name = None
        a = []
        with self.wiki_data_file as f:
            for line in f:
                line = line.replace('\n', '')
                if u'\t' not in line:
                    continue
                if name and name != line.split('\t')[0]:
                    self._check(name, a)
                    a = []
                name = line.split('\t')[0]
                a.append(line.split('\t')[1])
        self.training_set = set()
        t_f = self.wiki_data_file
        self.wiki_data_file = codecs.open(t_f.name, t_f.mode, 'utf-8')
        with self.wiki_data_file as f:
            for line in f:
                line = line.replace('\n', '')
                if u'\t' not in line:
                    continue
                if name and name != line.split('\t')[0]:
                    self.build_training_set(name, a)
                    a = []
                name = line.split('\t')[0]
                a.append(line.split('\t')[1])
        file_path = os.path.join(
            self.data_directory, 'categories.json')
        with codecs.open(file_path, 'w', 'utf-8') as f:
            f.write(json.dumps(self.categories))
        file_path = os.path.join(
            self.data_directory, 'training_set.dat')
        with codecs.open(file_path, 'w', 'utf-8') as f:
            f.write(str(self.training_set))



import math
import random
import codecs
import os

try:
    basestring
except:
    basestring = str


class Kian(object):
    """Train the model"""
    def __init__(self, model=None, lambda_para=0.0005, no_iter=250,
                 alpha=0.1, epi=0.2, training_set=None, slow=False,
                 skewed=False, balance_with_replacement=False):
        if not model and not training_set:
            raise ValueError('You must define a model or training set')
        self.model = model
        self.lambda_para = lambda_para
        self.no_iter = no_iter
        self.alpha = alpha
        self.epi = epi
        self.J_cv_history = []
        self.lambda_history = []
        self.theta_history = []
        self.J_history = []
        self.delta = []
        self.slow = slow
        if not training_set:
            tr_path = os.path.join(self.model.data_directory,
                                   'training_set.dat')
            with codecs.open(tr_path, 'r', 'utf-8') as f:
                training_set_temp = eval(f.read())
        else:
            training_set_temp = training_set
        self.training_set = []
        if skewed:
            if balance_with_replacement:
                training_set_temp = self.balance_training_set_replacement(
                    training_set_temp)
            else:
                training_set_temp = self.balance_training_set(
                    training_set_temp)
        for case in training_set_temp:
            if len(case) == 2 and hasattr(case[0], '__iter__'):
                    case = case[0] + [case[1]]
            self.training_set.append([self.value_normalizer(i) for i in case])
        if isinstance(self.training_set[0][-1], int):
            y_k = 1
        else:
            y_k = len(self.training_set[0][-1])
        self.arch = [len(self.training_set[0]), y_k]
        self.arch2 = [len(self.training_set[0]), len(self.training_set[0])]
        random.shuffle(self.training_set)
        split_number = int(len(self.training_set) / 5)
        self.cv_set = self.training_set[:split_number]
        self.training_set = self.training_set[split_number:]
        if len(self.training_set) > 30000 and not slow:
            self.training_set = random.sample(self.training_set, 30000)
        self.size = len(self.training_set)

    @staticmethod
    def balance_training_set(tr_set):
        no_outputs = {}
        tr_set2 = {}
        for case in tr_set:
            no_outputs[case[-1]] = no_outputs.get(case[-1], 0) + 1
            tr_set2[case[-1]] = tr_set2.get(case[-1], []) + [case]
        lowest_no = min(no_outputs.values())
        training_set = []
        for out in tr_set2:
            if no_outputs[out] > lowest_no:
                training_set += random.sample(tr_set2[out], lowest_no)
            else:
                training_set += tr_set2[out]

        return training_set

    @staticmethod
    def balance_training_set_replacement(tr_set):
        no_outputs = {}
        tr_set2 = {}
        for case in tr_set:
            no_outputs[case[-1]] = no_outputs.get(case[-1], 0) + 1
            tr_set2[case[-1]] = tr_set2.get(case[-1], []) + [case]
        highest_no = max(no_outputs.values())
        training_set = []
        for out in tr_set2:
            if no_outputs[out] < highest_no:
                training_set += [random.sample(tr_set2[out], 1)[0]
                                 for i in range(highest_no)]
            else:
                training_set += tr_set2[out]

        return random.sample(training_set, len(training_set))

    @staticmethod
    def value_normalizer(value):
        if value is True or value == "True":
            return 1
        if value is False or value == "False":
            return 0
        if isinstance(value, int) or isinstance(value, float):
            return value
        if isinstance(value, str):
            return float(value)
        res = []
        for case in value:
            res.append(Kian.value_normalizer(case))
        return res

    @staticmethod
    def forward(a, theta):
        sum_i = []
        for i in range(len(theta)):
            sum_i_2 = 0
            for j in range(len(a)):
                sum_i_2 += a[j] * theta[i][j]
            sum_i.append(sum_i_2)
        return sum_i

    @staticmethod
    def backward(delta, theta, a):
        res = []
        for i in range(len(theta[0])):
            sum_i = 0.0
            for j in range(len(theta)):
                sum_i += theta[j][i] * delta[j] * a[i] * (1 - a[i])
            res.append(sum_i)
        return res

    @staticmethod
    def sigmoid(a):
        return 1.0 / (1 + math.exp(-1 * a))

    @staticmethod
    def kian(theta, case):
        z = [[]] * 3
        a = [[]] * 3
        a[0] = [1] + case
        z[1] = Kian.forward(a[0], theta[0])
        a[1] = [1] + [Kian.sigmoid(i) for i in z[1]][1:]
        z[2] = Kian.forward(a[1], theta[1])
        a[2] = [Kian.sigmoid(i) for i in z[2]]
        return a[2]

    @staticmethod
    def cost_function(theta, training_set, reg=True, lambda_para=1):
        sum_cos = 0
        m = len(training_set)
        for case in training_set:
            y = Kian.kian(theta, case[:-1])
            if isinstance(case[-1], int):
                case[-1] = [case[-1]]
            for i in range(len(y)):
                sum_cos -= case[-1][i] * math.log(y[i]) + \
                    (1 - case[-1][i]) * math.log(1 - y[i])
        sum_cos = sum_cos / m
        if reg:
            for i in range(2):
                for j in range(len(theta[i])):
                    for k in range(len(theta[i][j])):
                        if not k or not j:
                            sum_cos += (lambda_para * theta[i][j][k] *
                                        theta[i][j][k]) / (2 * m)
        return sum_cos

    def train(self):
        print("Working on a training set size of %d" % self.size)

        while True:
            if len(self.J_cv_history) > 1 and self.J_cv_history[-1] > \
                    self.J_cv_history[-2]:
                break
            self.lambda_para *= 1.62
            print(self.lambda_para)
            theta = [[]] * 2
            self.J_history = []
            for i in range(2):
                theta[i] = [[]] * self.arch[i]
                for j in range(len(theta[i])):
                    theta[i][j] = [[]] * self.arch2[i]
                    for k in range(len(theta[i][j])):
                        theta[i][j][k] = (random.random() * self.epi * 2) \
                            - self.epi
            for i in range(self.no_iter):
                J = self.cost_function(theta, self.training_set,
                                       lambda_para=self.lambda_para)
                self.J_history.append(J)
                Delta = [[]] * 2
                for ii in range(2):
                    Delta[ii] = [[]] * self.arch[ii]
                    for j in range(len(Delta[ii])):
                        Delta[ii][j] = [[]] * self.arch2[ii]
                        for k in range(len(Delta[ii][j])):
                            Delta[ii][j][k] = 0
                D = Delta[:]
                for case in self.training_set:
                    z = [[]] * 3
                    a = [[]] * 3
                    delta = [[]] * 3
                    a[0] = [1] + case[:-1]
                    z[1] = self.forward(a[0], theta[0])
                    a[1] = [1] + [self.sigmoid(ii) for ii in z[1]][1:]
                    z[2] = self.forward(a[1], theta[1])
                    a[2] = [self.sigmoid(ii) for ii in z[2]]
                    delta[2] = [a[2][ii] - case[-1][ii] for ii
                                in range(len(case[-1]))]
                    delta[1] = self.backward(delta[2], theta[1], a[1])
                    for ii in range(2):
                        for j in range(len(Delta[ii])):
                            for k in range(len(Delta[ii][j])):
                                Delta[ii][j][k] += delta[ii + 1][j] * a[ii][k]
                for ii in range(2):
                    for j in range(len(Delta[ii])):
                        for k in range(len(Delta[ii][j])):
                            D[ii][j][k] = (1.0 / self.size) * Delta[ii][j][k]
                            if not j or not k:
                                D[ii][j][k] += self.lambda_para * \
                                    theta[ii][j][k]
                for ii in range(2):
                    for j in range(len(Delta[ii])):
                        for k in range(len(Delta[ii][j])):
                            theta[ii][j][k] -= self.alpha * D[ii][j][k]
            self.J_cv_history.append(
                self.cost_function(theta, self.cv_set, False))
            self.lambda_history.append(self.lambda_para)
            self.theta_history.append(theta)
        self.theta = theta
        self.D = D

    def finalize(self, d_path=None):
        d_theta = 0.00001
        theta_1 = self.theta[:]
        theta_2 = self.theta[:]
        theta_1[0][1][1] += d_theta
        dl1 = self.cost_function(theta_1, self.training_set)
        theta_2[0][1][1] -= 2 * d_theta
        dl2 = self.cost_function(theta_2, self.training_set)
        print("difference between results...")
        print(self.D[0][1][1], (dl1 - dl2) / (2 * d_theta))
        res_the = {0: [], 1: []}
        for case in self.training_set:
            res_the[case[-1][0]].append(self.kian(self.theta, case[:-1])[0])
        res_the2 = {0: [], 1: []}
        for case in self.cv_set:
            res_the2[case[-1][0]].append(self.kian(self.theta, case[:-1])[0])
        print("Cost function convergence")
        print(self.J_history[:10], self.J_history[-10:])
        if not d_path:
            d_path = self.model.data_directory
        with codecs.open(os.path.join(d_path, 'res1.dat'), 'w', 'utf-8') as f:
            f.write(str(res_the))
        with codecs.open(os.path.join(d_path, 'res2.dat'), 'w', 'utf-8') as f:
            f.write(str(res_the2))
        with codecs.open(os.path.join(d_path, 'theta.dat'), 'w', 'utf-8') as f:
            f.write(str(self.theta))

    def run(self):
        self.train()
        self.finalize()





def _ratio(test_set, thrashhold):
    false_positives = 0
    for case in test_set[0]:
        if case > thrashhold:
            false_positives += 1
    true_positives = 0
    for case in test_set[1]:
        if case > thrashhold:
            true_positives += 1
    return (false_positives / float(len(test_set[0])),
            true_positives / float(len(test_set[1])))


def AUC(test_set, step=0.01):
    thrashhold = 0
    auc = 0
    x_before = 1
    while thrashhold < 1:
        x, y = _ratio(test_set, thrashhold)
        auc += (x_before - x) * y
        x_before = x
        thrashhold += step
    return auc


def precision(a, b):
    true_positives = 0
    positives = 0
    for case in a[0]:
        if case > b:
            positives += 1
    for case in a[1]:
        if case > b:
            positives += 1
            true_positives += 1
    if positives == 0:
        return
    return float(true_positives) / float(positives)


def recall(a, b):
    true_positives = 0
    positives = 0
    for case in a[1]:
        if case > b:
            true_positives += 1
        positives += 1
    if positives == 0:
        return
    return float(true_positives) / float(positives)


def optimum_thrashhold(test_set, beta=0.5):
    thrashhold = 0
    res = {}
    x = []
    y = []
    while thrashhold < 1:
        pre = precision(test_set, thrashhold)
        rec = recall(test_set, thrashhold)
        if pre is None or rec is None:
            thrashhold += 0.001
            continue
        res[thrashhold] = (pre, rec)
        x.append(thrashhold)
        y.append((2 + (beta**2)) * (pre * rec) / ((pre * beta * beta) + rec))
        thrashhold += 0.001
    the_thrashhold = max(y)
    index = y.index(the_thrashhold)
    return x[index], res[x[index]]



import argparse

try:
    basestring
except NameError:
    raw_input = input

if __name__ == '__main__' and __package__ is None:
    from os import sys, path
    sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))

from kian import model

parser = argparse.ArgumentParser(description='Initiate a model.')
parser.add_argument('--name', '-n', nargs='?',
                    help='name of the model. No space character!')
parser.add_argument('--wiki', '-w', nargs='?',
                    help='code of Wiki, e.g. enwiki, fawikivoyage')
parser.add_argument('--prop', '-p', nargs='?',
                    help='Number or id of property, e.g. P31, P17')
parser.add_argument('--value', '-v', nargs='?',
                    help='value of the property. e.g. Q31, Q183')

args = parser.parse_args()
if args.name:
    name = args.name
else:
    name = str(
        raw_input('Please provide a name. No space character:')).strip()
if args.wiki:
    wiki = args.wiki
else:
    wiki = str(raw_input('Code of Wiki, e.g. enwiki, fawikivoyage:')).strip()
if args.prop:
    property_name = args.prop
else:
    property_name = str(
        raw_input('Number or id of property, e.g. P31, P17:')).strip()
if args.value:
    value = args.value
else:
    value = str(raw_input('value of the property. e.g. Q31, Q183:')).strip()

if ' ' in name:
    name = name.replace(' ', '_')
if 'wiki' not in wiki:
    raise ValueError('Name of wiki is not valid.')
if not property_name.startswith('P'):
    try:
        int(property_name)
    except:
        raise ValueError('Invalid property name')
    else:
        property_name = 'P' + property_name
try:
    int(property_name[1:])
except:
    raise ValueError('Invalid property name')
if not value.startswith('Q'):
    try:
        int(value)
    except:
        raise ValueError('Invalid value')
    else:
        property_name = 'Q' + property_name
try:
    int(property_name[1:])
except:
    raise ValueError('Invalid value')

# TODO: Further settings.

model = model.Model(name=name, wiki=wiki, property_name=property_name,
                    value=value)
try:
    model.write_file()
except RuntimeError:
    print('The model with the same name already exists')
    choice = str(input('Overwrite?[Y]es, [N]ew name:'))
    if choice.lower().strip() == 'y':
        model.write_file(force=True)
    elif choice.lower.strip() == 'n':
        choice = str(input('New name:'))
        model.name = choice.strip().replace(' ', '_')
        model.write_model()
    else:
        raise ValueError('Not a valid option')



import argparse
import pywikibot
import sys
import codecs
import os

try:
    basestring
except NameError:
    import pymysql as MySQLdb
else:
    import MySQLdb

if __name__ == '__main__' and __package__ is None:
    from os import sys, path
    sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))

import kian.fitness
from kian import TrainedModel
from kian import Kian

site = pywikibot.Site('wikidata', fam='wikidata')
repo = site.data_repository()


def store(name, res, wiki, p_number, value):
    item = pywikibot.ItemPage(repo, name)
    try:
        item.get()
    except pywikibot.NoPage:
        return False
    if wiki not in item.sitelinks:
        return False
    insert_statement = (
        "INSERT INTO kian_mistakes "
        "(qid, property, value, wiki_name, name_wiki, status, prob, wd_value) "
        "VALUES (%s, %s, %s, %s, %s, %s, %s, %s)")
    db = MySQLdb.connect(host="tools-db", db="s52709__kian_p",
                         read_default_file="~/replica.my.cnf", charset='utf-8')
    cursor = db.cursor()
    name = int(name.lower().replace('q', ''))
    wd_value = 1 if res < 0.5 else 0
    cursor.execute(insert_statement, (name, p_number, value, wiki,
                                      item.sitelinks[wiki], 0, res, wd_value))
    db.commit()
    cursor.close()
    db.close()
    print('Stored')
    return True


def main():
    args = sys.argv[1:]
    parser = argparse.ArgumentParser(description='Parse and add statements '
                                     'based on trained model')
    parser.add_argument('--name', '-n', nargs='?', required=True,
                        help='name of the model to train')
    parsed_args = parser.parse_known_args(args)[0]

    model = TrainedModel.from_file(parsed_args.name)
    print('Loading the model')
    model.load_data()
    model.load()
    file_path = os.path.join(model.data_directory, 'res2.dat')

    if not os.path.isfile(file_path):
        raise ValueError('You should train the model first')
    with codecs.open(file_path, 'r', 'utf-8') as f:
        cv_set = eval(f.read())
    second_thrashhold = kian.fitness.optimum_thrashhold(cv_set, 0.125)[0]
    print('Finding possible mistakes')
    print(second_thrashhold)
    name = None
    a = []
    p_number = int(model.property_name.lower().replace('p', ''))
    q_number = int(model.value.lower().replace('q', ''))
    with model.wiki_data_file as f:
        for line in f:
            line = line.replace('\n', '')
            if u'\t' not in line:
                continue
            if name and name != line.split('\t')[0]:
                features = model.label_case(a)
                res = Kian.kian(model.theta, features)[0]
                if name in model.wikidata_data_w and \
                        res < (1 - second_thrashhold):
                    store(name, res, model.wiki, p_number, q_number)
                elif name in model.wikidata_data_wo and \
                        res > second_thrashhold:
                    store(name, res, model.wiki, p_number, q_number)
                a = []
            name = line.split('\t')[0]
            a.append(line.split('\t')[1])


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        sys.exit()



import argparse
import pywikibot
import sys
import codecs
import os
if __name__ == '__main__' and __package__ is None:
    from os import sys, path
    sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))

import kian.fitness
from kian import TrainedModel
from kian import Kian

site = pywikibot.Site('wikidata', fam='wikidata')


def main():
    res_text = u"{| class=\"wikitable sortable\"\n!Qid!!Value<br />" \
        u"(Wikidata)!!Value<br />(Wipedia)!!Other boring stuff"
    args = sys.argv[1:]
    parser = argparse.ArgumentParser(description='Parse and add statements '
                                     'based on trained model')
    parser.add_argument('--name', '-n', nargs='?', required=True,
                        help='name of the model to train')
    parsed_args = parser.parse_known_args(args)[0]

    model = TrainedModel.from_file(parsed_args.name)
    print('Loading the model')
    model.load_data()
    model.load()
    file_path = os.path.join(model.data_directory, 'res2.dat')

    if not os.path.isfile(file_path):
        raise ValueError('You should train the model first')
    with codecs.open(file_path, 'r', 'utf-8') as f:
        cv_set = eval(f.read())
    second_thrashhold = kian.fitness.optimum_thrashhold(cv_set, 0.125)[0]
    print('Finding possible mistakes')
    print(second_thrashhold)
    name = None
    a = []
    with model.wiki_data_file as f:
        for line in f:
            line = line.replace('\n', '')
            if u'\t' not in line:
                continue
            if name and name != line.split('\t')[0]:
                features = model.label_case(a)
                res = Kian.kian(model.theta, features)[0]
                if name in model.wikidata_data_w and \
                        res < (1 - second_thrashhold):
                    res_text += ("\n|-\n|[[%s]]||Yes||No (%s)||%s" %
                                 (name, res, features))
                elif name in model.wikidata_data_wo and \
                        res > second_thrashhold:
                    res_text += ("\n|-\n|[[%s]]||No||Yes (%s)||%s" %
                                 (name, res, features))
                a = []
            name = line.split('\t')[0]
            a.append(line.split('\t')[1])
    page_title = "User:Ladsgroup/Kian/Possible mistakes/%s" % model.name
    page = pywikibot.Page(site, page_title)
    page.put(res_text + "\n|}", "Bot: Report")
if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        sys.exit()



import json
import os
import codecs
from kian.parser import ModelWithData


class TrainedModel(ModelWithData):
    """docstring for TrainedModel"""
    def __init__(self, *args, **kwargs):
        super(TrainedModel, self).__init__(*args, **kwargs)
        file_path = os.path.join(
            self.data_directory, 'categories.json')
        if not os.path.isfile(file_path):
            raise ValueError('You should train the model first')
        file_path = os.path.join(
            self.data_directory, 'theta.dat')
        if not os.path.isfile(file_path):
            raise ValueError('You should train the model first')

    def load_categories(self):
        file_path = os.path.join(
            self.data_directory, 'categories.json')
        with codecs.open(file_path, 'r', 'utf-8') as f:
            self.categories = json.loads(f.read())

    def load_theta(self):
        file_path = os.path.join(
            self.data_directory, 'theta.dat')
        with codecs.open(file_path, 'r', 'utf-8') as f:
            self.theta = eval(f.read())

    def load(self):
        self.load_categories()
        self.load_theta()



import argparse
import codecs
import os

if __name__ == '__main__' and __package__ is None:
    from os import sys, path
    sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))

import kian.fitness
from kian import TrainedModel

parser = argparse.ArgumentParser(description='Evaluate a trained model')
parser.add_argument('--name', '-n', nargs='?', required=True,
                    help='name of the model to train')
args = parser.parse_args()

model = TrainedModel.from_file(args.name)
file_path = os.path.join(model.data_directory, 'res2.dat')

if not os.path.isfile(file_path):
    raise ValueError('You should train the model first')
with codecs.open(file_path, 'r', 'utf-8') as f:
    cv_set = eval(f.read())
AUC = kian.fitness.AUC(cv_set)

first_thrashhold = kian.fitness.optimum_thrashhold(cv_set, 1)
second_thrashhold = kian.fitness.optimum_thrashhold(cv_set, 0.5)
third_thrashhold = kian.fitness.optimum_thrashhold(cv_set, 0.25)

print('AUC of the classifier: {0:.4}'.format(AUC))
print('First thrashhold (recall and precision): %s (%s, %s)' %
      (first_thrashhold[0], first_thrashhold[1][0], first_thrashhold[1][1]))
print('Second thrashhold (recall and precision): %s (%s, %s)' %
      (second_thrashhold[0], second_thrashhold[1][0],
       second_thrashhold[1][1]))
print('Third thrashhold (recall and precision): %s (%s, %s)' %
      (third_thrashhold[0], third_thrashhold[1][0], third_thrashhold[1][1]))



import argparse
import pywikibot
import sys
import codecs
import os
from pywikibot import pagegenerators
if __name__ == '__main__' and __package__ is None:
    from os import sys, path
    sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))

import kian.fitness
from kian import TrainedModel
from kian import Kian
sources = {
    'enwiki': 'Q328',
    'svwiki': 'Q169514',
    'dewiki': 'Q48183',
    'itwiki': 'Q11920',
    'nowiki': 'Q191769',
    'fawiki': 'Q48952',
    'eswiki': 'Q8449',
    'plwiki': 'Q1551807',
    'cawiki': 'Q199693',
    'frwiki': 'Q8447',
    'nlwiki': 'Q10000',
    'ptwiki': 'Q11921',
    'ruwiki': 'Q206855',
    'viwiki': 'Q200180',
    'bewiki': 'Q877583',
    'ukwiki': 'Q199698',
    'trwiki': 'Q58255',
    'cswiki': 'Q191168',
    'shwiki': 'Q58679',
    'jawiki': 'Q177837',
    'kowiki': 'Q17985',
    'fiwiki': 'Q175482',
    'huwiki': 'Q53464',
    'rowiki': 'Q199864',
    'zhwiki': 'Q30239',
    'ltwiki': 'Q202472',
    'srwiki': 'Q200386',
    'arwiki': 'Q199700',
    'skwiki': 'Q192582',
    'elwiki': 'Q11918',
    'hrwiki': 'Q203488',
    'mswiki': 'Q845993',
    'glwiki': 'Q841208',
    'euwiki': 'Q207260'}


def main():
    args = sys.argv[1:]
    parser = argparse.ArgumentParser(description='Parse and add statements '
                                     'based on trained model')
    parser.add_argument('--name', '-n', nargs='?', required=True,
                        help='name of the model to train')
    parsed_args = parser.parse_known_args(args)[0]

    model = TrainedModel.from_file(parsed_args.name)
    model.load()
    file_path = os.path.join(model.data_directory, 'res2.dat')

    if not os.path.isfile(file_path):
        raise ValueError('You should train the model first')
    with codecs.open(file_path, 'r', 'utf-8') as f:
        cv_set = eval(f.read())
    second_thrashhold = kian.fitness.optimum_thrashhold(cv_set, 0.25)[0]

    local_args = pywikibot.handle_args(args)
    genFactory = pagegenerators.GeneratorFactory()
    for arg in local_args:
        genFactory.handleArg(arg)
    generator = genFactory.getCombinedGenerator()
    repo = pywikibot.Site().data_repository()
    for page in generator:
        cats = [cat.title(underscore=True, withNamespace=False)
                for cat in page.categories()]
        features = model.label_case(cats)
        res = Kian.kian(model.theta, features)[0]
        if res > second_thrashhold:
            try:
                item = pywikibot.ItemPage.fromPage(page)
            except pywikibot.NoPage:
                continue
            if model.property_name in item.claims:
                continue
            claim = pywikibot.Claim(repo, model.property_name)
            claim.setTarget(pywikibot.ItemPage(repo, model.value))
            item.addClaim(claim, summary='Bot: Adding %s:%s from %s '
                          '([[User:Ladsgroup/Kian|Powered by Kian]])' %
                          (model.property_name, model.value, model.wiki))
            source = pywikibot.Claim(repo, 'P143')
            source.setTarget(pywikibot.ItemPage(repo, sources[model.wiki]))
            claim.addSource(source)

if __name__ == "__main__":
    main()



import argparse
import pywikibot
import sys
import codecs
import os
from pywikibot import pagegenerators
if __name__ == '__main__' and __package__ is None:
    from os import sys, path
    sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))

import kian.fitness
from kian import TrainedModel
from kian import Kian


def main():
    args = sys.argv[1:]
    parser = argparse.ArgumentParser(description='Parse and add statements '
                                     'based on trained model')
    parser.add_argument('--name', '-n', nargs='?', required=True,
                        help='name of the model to train')
    parsed_args = parser.parse_known_args(args)[0]

    model = TrainedModel.from_file(parsed_args.name)
    model.load()
    file_path = os.path.join(model.data_directory, 'res2.dat')

    if not os.path.isfile(file_path):
        raise ValueError('You should train the model first')
    with codecs.open(file_path, 'r', 'utf-8') as f:
        cv_set = eval(f.read())
    first_thrashhold = kian.fitness.optimum_thrashhold(cv_set, 1)[0]
    second_thrashhold = kian.fitness.optimum_thrashhold(cv_set, 0.5)[0]
    second_thrashhold = kian.fitness.optimum_thrashhold(cv_set, 0.25)[0]
    pywikibot.output('1st, 2nd, and 3rd thrashholds are: %s, %s, %s' %
                     (first_thrashhold, second_thrashhold, second_thrashhold))
    local_args = pywikibot.handle_args(args)
    genFactory = pagegenerators.GeneratorFactory()
    for arg in local_args:
        genFactory.handleArg(arg)
    generator = genFactory.getCombinedGenerator()
    for page in generator:
        cats = [cat.title(underscore=True, withNamespace=False)
                for cat in page.categories()]
        features = model.label_case(cats)
        res = Kian.kian(model.theta, features)[0]
        pywikibot.output(page.title(), res)


if __name__ == "__main__":
    main()



import json
import codecs
import os


class Model(object):
    """docstring for Model"""
    def __init__(self, name, wiki, property_name, value, bias_unit=None):
        self.name = name
        self.wiki = wiki
        self.property_name = property_name
        self.value = value
        path = os.path.dirname(os.path.realpath(__file__))
        self.data_directory = os.path.abspath(
            os.path.join(path, os.pardir, 'data',
                         '%s' % self.name))
        if bias_unit:
            self.bias_unit = bias_unit

    @classmethod
    def from_file(cls, name):
        path = os.path.dirname(os.path.realpath(__file__))
        file_path = os.path.abspath(
            os.path.join(path, os.pardir, 'models',
                         '%s.json' % name))
        if not os.path.isfile(file_path):
            raise RuntimeError('File could not be found!')
        with codecs.open(file_path, 'r', 'utf-8') as f:
            content = json.loads(f.read())
        model = cls(**content)
        return model

    def write_file(self, force=False):
        path = os.path.dirname(os.path.realpath(__file__))
        file_path = os.path.abspath(
            os.path.join(path, os.pardir, 'models',
                         '%s.json' % self.name))
        print(file_path)
        if os.path.isfile(file_path) and not force:
            raise RuntimeError('File is there!')
        content = {}
        content['name'] = self.name
        content['wiki'] = self.wiki
        content['property_name'] = self.property_name
        content['value'] = self.value
        with codecs.open(file_path, 'w', 'utf-8') as f:
            f.write(json.dumps(content))



# -*- coding: utf-8  -*-
"""Installer script for Kian."""
#
# (C) Amir Sarabadani
#
# Distributed under the terms of the MIT license.
#
import os

from setuptools import find_packages, setup


def requirements(fname):
    return [line.strip()
            for line in open(os.path.join(os.path.dirname(__file__), fname))]


setup(
    name='kian',
    version='0.2.0',
    description='Kian is the neural network designed to serve Wikidata.',
    long_description=open('README.rst').read(),
    maintainer='Amir Sarabadani',
    maintainer_email='ladsgroup@gmail.com',
    license='MIT License',
    install_requires=requirements("requirements.txt"),
    packages=find_packages(),
    url='https://github.com/Ladsgroup/Kian',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Development Status :: 3 - Alpha',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Environment :: Console',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
    ],
)
